@@ -3,7 +3,7 @@
  * Copyright 1994-1999 Patrick Powell, San Diego, CA <papowell@astart.com>
  **************************************************************************/
 /**** HEADER *****
-$Id: ifhp.h,v 1.41 2003/09/04 18:38:32 papowell Exp papowell $
+$Id: ifhp.h,v 1.47 2004/09/30 23:24:41 papowell Exp papowell $
  **** ENDHEADER ****/
 
 #ifndef _IFHP_H_
@@ -133,31 +133,32 @@ EXTERN char *MSG DEFINE( = "MSG");
     struct glyph *glyph;    /* glyphs */
 };
 
-EXTERN char *Devfd0  DEFINE( = DEVFD0 );
-EXTERN char *InputFile;
+EXTERN char *InputFile DEFINE( = "-" );
 
 /*
  * setup values
  */
 
 EXTERN OBJ *Zopts, *Topts, *Unsorted_Zopts, *Unsorted_Topts,
-	*Raw, *Model, *Index, *Entries, *Devstatus, *Pjl_only, *Pjl_except,	/* option variables */
+	*Raw, *Model, *Printcap, *Index, *Entries, *Devstatus, *Pjl_only, *Pjl_except,	/* option variables */
 	*Pjl_options_set, *Pjl_show_values,
 	*Pjl_vars_set, *Pjl_vars_except,
 	*Pcl_vars_set, *Pcl_vars_except, *Pcl_papersize_codes,
 	*User_opts, *Pjl_user_opts, *Pcl_user_opts, *Ps_user_opts,
-	*Pjl_error_codes, *Pjl_quiet_codes, *Pjl_alert_codes, *Setvals;
+	*Pjl_error_codes, *Pjl_quiet_codes, *Pjl_alert_codes, *Setvals,
+	*Ignore, *Translate, *Status_fields;
 
 EXTERN OBJ **Var_list[]
 #ifdef DEF
  = {
 	&Zopts, &Topts, &Unsorted_Zopts, &Unsorted_Topts,
-	&Raw, &Model, &Index, &Entries,  &Devstatus, &Pjl_only, &Pjl_except,	/* option variables */
+	&Raw, &Model, &Printcap, &Index, &Entries,  &Devstatus, &Pjl_only, &Pjl_except,	/* option variables */
 	&Pjl_options_set, &Pjl_show_values,
 	&Pjl_vars_set, &Pjl_vars_except,
 	&Pcl_vars_set, &Pcl_vars_except, &Pcl_papersize_codes,
 	&User_opts, &Pjl_user_opts, &Pcl_user_opts, &Ps_user_opts,
 	&Pjl_error_codes, &Pjl_quiet_codes, &Pjl_alert_codes, &Setvals,
+	&Ignore, &Translate, &Status_fields,
 	0
 }
 #endif
@@ -170,16 +171,16 @@ EXTERN char **Argv;			/* parms variables */
 EXTERN int Argc;			/* we have the number of variables */
 EXTERN time_t Start_time;     /* start time of program */
 EXTERN char RemoteIPAddr[256];
-EXTERN char *Printcap_lp;
 
 EXTERN int
 	Appsocket,	/* accounting fd */
 	Autodetect,	/* let printer autodetect type */
-	Close_connection, /* close the appsocket connection */
+	Close_appsocket, /* close the appsocket connection */
 	Crlf,		/* only do CRLF */
 	Dev_retries,	/* number of retries on open */
 	Dev_sleep DEFINE(=1000),	/* wait between retries in Millisec */
 	Errorcode,		/* exit value */
+	Foomatic DEFINE(=1),	/* enable foomatic conversion */
 	Force_conversion,	/* force conversion by file utility */
 	Force_processing,	/* force processing by ifhp */
 	Force_status,	/* even if device is not socket or tty, allow status */
@@ -228,6 +229,9 @@ EXTERN int
 	Status_fd DEFINE(=-2),	/* status reporting to remote site */
 	Sync_interval,	/* sync interval */
 	Sync_timeout,	/* sync timeout */
+	Snmp_monitor,		/* use snmp monitor */
+	Snmp_wait_after_close DEFINE(=1),	/* use snmp monitor */
+	Snmp_fd,			/* read status from snmp monitor */
 	Tbcp,		/* supports Postscript TBCP */
 	Text,		/* supports test files */
 	Trace_on_stderr, /* puts out trace on stderr as well */
@@ -244,14 +248,23 @@ EXTERN char
 	*Config_file,	/* config file list */
 	*Device,		/* device to open */
 	*End_status,	/* status file */
+	*Foomatic_rip,	/* foomatic rip */
 	*Model_id,		/* printer model */
 	*Pagecount,		/* pagecount */
 	*Name,			/* program name */
 	*Pjl_ready_msg,		/* ready message for console */
 	*Pjl_done_msg,		/* done message for console */
+	*Ppd_file,			/* PPD file */
 	*Remove_ctrl,	/* remove these control chars */
 	*Statusfile,	/* status file */
+	*Status_ignore,	/* ignore these status entries */
+	*Status_translate,	/* ignore these status entries */
 	*Stty_args,		/* if device is tty, stty values */
+	*Snmp_dev,	/* SNMP device */
+	*Snmp_program,	/* SNMP program */
+	*Snmp_status_fields,	/* SNMP status fields */
+	*Snmp_sync_status,	/* SNMP sync status */
+	*Snmp_end_status,	/* SNMP end status */
 	*Sync,			/* synchronize printer */
 	*Waitend;		/* wait for end using sync */
 
@@ -298,12 +311,13 @@ void Pr_status( char *str );
 void Check_device_status( char *line, int infovar );
 void Initialize_parms( OBJ *list, struct keyvalue *valuelist );
 void Dump_parms( char *title, struct keyvalue *v );
-void Process_job( int do_pagecount, int pagecount_ps, int pagecount_pjl, char *pagecount_prog );
+void Process_job( int do_pagecount, int pagecount_ps, int pagecount_pjl,
+	char *pagecount_prog, int pagecount_snmp );
 void Start_of_job( int *startpagecounter, int do_pagecount, int pagecount_ps,
-	int pagecount_pjl, int nested_job, char *pagecount_prog );
+	int pagecount_pjl, int nested_job, char *pagecount_prog, int pagecount_snmp );
 void End_of_job( int *startpagecounter, int do_pagecount, int pagecount_ps,
 	int pagecount_pjl, int wait_for_end, int nested_job, int banner_page,
-	char *pagecount_prog );
+	char *pagecount_prog, int pagecount_snmp );
 int Find_in_list( OBJ *list, const char *str );
 void Put_pjl( char *s );
 void Put_pcl( char *s );
@@ -348,10 +362,10 @@ int Pcl_setvar(char *prefix, char*id, char *value, Wr_out routine );
 void Do_sync( int sync_timeout, int sync_interval, int pagecount_pjl );
 void Do_waitend( int waitend_timeout, int waitend_interval,
 	int waitend_ctrl_t_interval, int banner );
-int Check_pagecount( int *use_ps_v, int *use_pjl_v, char **use_prog_v );
+int Check_pagecount( int *use_ps_v, int *use_pjl_v, char **use_prog_v, int *use_snmp_v );
 int Do_pagecount( int pagecount_timeout, int pagecount_interval, int pagecount_poll,
-	 int use_ps, int use_pjl, char *use_prog );
-int Current_pagecounter( int pagecount_timeout, int use_pjl, int use_ps );
+	 int use_ps, int use_pjl, char *use_prog, int use_snmp );
+int Current_pagecounter( int pagecount_timeout, int use_pjl, int use_ps, int use_snmp );
 void Send_job();
 void URL_decode( char *s );
 int Process_OF_mode(void);
@@ -359,7 +373,6 @@ void close_on_exec( int n );
 void Set_max_fd( int n );
 void Use_file_util(char *pgm, char *value, int value_len );
 int Make_tempfile( void );
-void Unlink_tempfile( char *filename );
 void Make_stdin_file();
 char * Set_mode_lang( char *s );
 int Fd_readable( int fd, int *poll_for_status );
