@@ -3,7 +3,7 @@
  * Copyright 1994-1999 Patrick Powell, San Diego, CA <papowell@astart.com>
  **************************************************************************/
 /**** HEADER *****/
-static char *const _id = "$Id: stty.c,v 1.1 1999/04/07 23:14:38 papowell Exp papowell $";
+ static char *const _id = "$Id: stty.c,v 1.6 2002/04/01 18:01:03 papowell Exp papowell $";
 
 #include "ifhp.h"
 
@@ -44,7 +44,7 @@ static char *const _id = "$Id: stty.c,v 1.1 1999/04/07 23:14:38 papowell Exp pap
 #endif  /* !defined(EXTB) */
 #endif  /* !defined(B38400) */
 
-static struct bauds {
+ static struct bauds {
     char *string;
     int baud;
     int speed;
@@ -72,11 +72,11 @@ static struct bauds {
  *  such as "pass8" and "parity".
  */
 
-struct tchars termctrl;
-struct ltchars linectrl;
-struct sgttyb mode;
+ struct tchars termctrl;
+ struct ltchars linectrl;
+ struct sgttyb mode;
 
-static struct {
+ static struct {
     char *string;
     int set;
     int reset;
@@ -150,7 +150,7 @@ static struct {
 };
 
 
-static struct special {
+ static struct special {
     char *name;
     char *cp;
     char def;
@@ -158,30 +158,33 @@ static struct special {
 
     { "stop", &termctrl.t_stopc, CSTOP },
     { "start", &termctrl.t_startc, CSTART },
-    { 0 }
+    { 0, 0, 0 }
 };
 
 void Do_stty( int fd, char *Stty_command )
 {
 	int i, localmode, linedisc, argcount;
 	char *s, *arg, *parm = 0;
-	struct line_list l;
+	OBJ *l = 0;
 
-	Init_line_list( &l );
-	Split( &l, Stty_command, Whitespace, 0, 0, 0, 1, 0 );
+	l = NEW_OBJ(0,MEMINFO);
+	l = Split_STR_OBJ( /*p*/l, /*nomod*/1, /*str*/Stty_command,
+			/*type*/OBJ_T_LIST, /*linesep*/WHITESPACE, /*escape*/0,
+			/*comment*/ 0, /*trim*/1, /*keysep*/0,/*do_append*/0, /*lc*/0,
+			/*flagvalues*/ 0, /*value_urlunesc*/0, MEMINFO );
 
-	DEBUG4("Do_Do_stty: using SGTTYB, fd %d", fd );
+	DEBUG4("Do_stty: using SGTTYB, fd %d", fd );
 	if( ioctl( fd, TIOCGETP, &mode) < 0
 		|| ioctl( fd, TIOCGETC, &termctrl) < 0
 		|| ioctl( fd, TIOCLGET, &localmode) < 0
 		|| ioctl( fd, TIOCGLTC, &linectrl) < 0 ){
 		Errorcode = JABORT;
-		logerr_die( "cannot get tty parameters");
+		LOGERR_DIE(LOG_INFO) "cannot get tty parameters");
 	}
 	DEBUG4("Do_stty: before mode 0x%x, lmode 0x%x, speed 0x%x",
 			 mode.sg_flags, localmode, mode.sg_ispeed);
-	for( argcount = 0; argcount < l.count; ++argcount ){
-		arg = l.list[argcount];
+	for( argcount = 0; argcount < LEN_LIST_OBJ(l); ++argcount ){
+		arg = GET_ENTRY_LIST_OBJ(l,argcount);
 		for( i = 0; modes[i].string && strcasecmp( modes[i].string, arg); i++);
 		if( modes[i].string ){
 			DEBUG4("Do_stty: modes %s, mc 0x%x ms 0x%x lc 0x%x ls 0x%x",
@@ -195,11 +198,11 @@ void Do_stty( int fd, char *Stty_command )
 		}
 		for( i = 0; special[i].name && strcasecmp( special[i].name, arg); i++);
 		if( special[i].name) {
-			if( ++argcount < l.count ){
-				parm = l.list[argcount];
+			if( ++argcount < LEN_LIST_OBJ(l) ){
+				parm = GET_ENTRY_LIST_OBJ(l,argcount);
 			} else {
 				Errorcode = JABORT;
-				fatal(  "Do_stty: missing parameter for %s\n", arg);
+				FATAL(LOG_INFO)  "Do_stty: missing parameter for %s\n", arg);
 			}
 			if( parm[0] == '^') {
 				if( parm[1] == '?') {
@@ -224,7 +227,7 @@ void Do_stty( int fd, char *Stty_command )
 			linedisc = NTTYDISC;
 			if( ioctl( fd, TIOCSETD, &linedisc) < 0){
 				Errorcode = JABORT;
-				logerr_die( "Do_stty: TIOCSETD ioctl failed");
+				LOGERR_DIE(LOG_INFO) "Do_stty: TIOCSETD ioctl failed");
 			}
 			continue;
 		}
@@ -233,11 +236,11 @@ void Do_stty( int fd, char *Stty_command )
 			linedisc = 0;
 			if( ioctl( fd, TIOCSETD, &linedisc) < 0){
 				Errorcode = JABORT;
-				logerr_die( "Do_stty: TIOCSETD ioctl failed");
+				LOGERR_DIE(LOG_INFO) "Do_stty: TIOCSETD ioctl failed");
 			}
 			continue;
 		}
-		logmsg("Do_stty: unknown mode: %s", arg);
+		LOGMSG(LOG_INFO)"Do_stty: unknown mode: %s", arg);
 	}
 	DEBUG4("Do_stty: after mode 0x%x, lmode 0x%x, speed 0x%x",
 			 mode.sg_flags, localmode, mode.sg_ispeed);
@@ -246,7 +249,7 @@ void Do_stty( int fd, char *Stty_command )
 		|| ioctl( fd, TIOCSLTC, &linectrl) < 0
 		|| ioctl( fd, TIOCLSET, &localmode) < 0) {
 		Errorcode = JABORT;
-		logerr_die( "cannot set tty parameters");
+		LOGERR_DIE(LOG_INFO) "cannot set tty parameters");
 	}
 	Free_line_list( &l );
 }
@@ -263,7 +266,7 @@ void Do_stty( int fd, char *Stty_command )
 #define TOSTOP _TOSTOP
 #endif
 
-static struct bauds {
+ static struct bauds {
     char *string;
     int baud;
     int speed;
@@ -284,8 +287,8 @@ static struct bauds {
     { (char *) 0, 0, 0 }
 };
 
-struct termio tio;
-static struct {
+ struct termio tio;
+ static struct {
     char *string;
     int iset;
     int ireset;
@@ -484,25 +487,28 @@ static struct {
     {0},
 };
 
-void Do_stty( int fd, char *Stty_command )
+ void Do_stty( int fd, char *Stty_command )
 {
 	int i, localmode, linedisc, argcount;
 	char *s, *arg, *parm = 0;
-	struct line_list l;
+	OBJ *l = 0;
 
-	Init_line_list( &l );
-	Split( &l, Stty_command, Whitespace, 0, 0, 0, 1, 0 );
+	l = NEW_OBJ(0,MEMINFO);
+	l = Split_STR_OBJ( /*p*/l, /*nomod*/1, /*str*/Stty_command,
+			/*type*/OBJ_T_LIST, /*linesep*/WHITESPACE, /*escape*/0,
+			/*comment*/ 0, /*trim*/1, /*keysep*/0,/*do_append*/0, /*lc*/0,
+			/*flagvalues*/ 0, /*value_urlunesc*/0, MEMINFO );
 
 
-	DEBUG4("Do_Do_stty: using TERMIO, fd %d", fd );
+	DEBUG4("Do_stty: using TERMIO, fd %d", fd );
 	if( ioctl( fd, TCGETA, &tio) < 0) {
 		Errorcode = JABORT;
-		logerr_die( "cannot get tty parameters");
+		LOGERR_DIE(LOG_INFO) "cannot get tty parameters");
 	}
 	DEBUG4("Do_stty: before imode 0x%x, omode 0x%x, cmode 0x%x, lmode 0x%x",
 			 tio.c_iflag, tio.c_oflag, tio.c_cflag, tio.c_lflag);
-	for( argcount = 0; argcount < l.count; ++argcount ){
-		arg = l.list[argcount];
+	for( argcount = 0; argcount < LEN_LIST_OBJ(l); ++argcount ){
+		arg = GET_ENTRY_LIST_OBJ(l,argcount);
 		for( i = 0; tmodes[i].string && strcasecmp( tmodes[i].string, arg); i++);
 
 		if( tmodes[i].string) {
@@ -528,14 +534,14 @@ void Do_stty( int fd, char *Stty_command )
 			tio.c_cflag |= bauds[i].speed;
 			continue;
 		}
-		logmsg("Do_stty: unknown mode: %s", arg);
+		LOGMSG(LOG_INFO)"Do_stty: unknown mode: %s", arg);
 	}
 
 	DEBUG4("Do_stty: before imode 0x%x, omode 0x%x, cmode 0x%x, lmode 0x%x",
 			 tio.c_iflag, tio.c_oflag, tio.c_cflag, tio.c_lflag);
 	if( ioctl( fd, TCSETA, &tio) < 0) {
 		Errorcode = JABORT;
-		logerr_die( "cannot set tty parameters");
+		LOGERR_DIE(LOG_INFO) "cannot set tty parameters");
 	}
 	Free_line_list(&l);
 }
@@ -549,7 +555,7 @@ void Do_stty( int fd, char *Stty_command )
 					/* ignore the flags arg. */
 #endif
 
-static struct bauds {
+ static struct bauds {
 	char *string;
 	int baud;
 	int speed;
@@ -575,7 +581,7 @@ static struct bauds {
 	{ (char *) 0, 0, 0 }
 };
 
-struct s_term_dat {
+ struct s_term_dat {
 	char *name;
 	uint or_dat;
 	uint and_dat;
@@ -584,7 +590,7 @@ struct s_term_dat {
 #define FLAGS(X) { #X, X , 0 }, { "-" #X, 0, X }
 
 /* c_iflag bits */
-static struct s_term_dat c_i_dat[] =
+ static struct s_term_dat c_i_dat[] =
 {
 	FLAGS(IGNBRK),
 	FLAGS(BRKINT),
@@ -615,7 +621,7 @@ static struct s_term_dat c_i_dat[] =
 };
 
 /* c_oflag bits */
-static struct s_term_dat c_o_dat[] =
+ static struct s_term_dat c_o_dat[] =
 {
 	FLAGS(OPOST),
 #ifdef OLCUC
@@ -677,7 +683,7 @@ static struct s_term_dat c_o_dat[] =
 };
 
 /* c_cflag bit meaning */
-static struct s_term_dat c_c_dat[] =
+ static struct s_term_dat c_c_dat[] =
 {
 	{"CS5", CS5, CSIZE},
 	{"CS6", CS6, CSIZE},
@@ -721,7 +727,7 @@ static struct s_term_dat c_c_dat[] =
 };
 
 /* c_lflag bits */
-static struct s_term_dat c_l_dat[] =
+ static struct s_term_dat c_l_dat[] =
 {
 	{"ISIG", ISIG, 0},
 	{"-ISIG", 0, ISIG},
@@ -783,46 +789,49 @@ static struct s_term_dat c_l_dat[] =
 	{ 0 ,0 ,0 }
 };
 
-struct termios t_dat;
+ struct termios t_dat;
 
-static struct special {
+ static struct special {
 	char *name;
 	char *cp;
 }       special[] = {
 
 	{ "stop", (char *)&(t_dat.c_cc[VSTOP]) },
 	{ "start", (char *)&(t_dat.c_cc[VSTART]) },
-	{ 0 }
+	{ 0, 0 }
 };
 
-void Do_stty( int fd, char *Stty_command )
+ void Do_stty( int fd, char *Stty_command )
 {
 	int i, argcount;
 	char *arg, *parm = 0;
-	struct line_list l;
+	OBJ *l = 0;
 
-	Init_line_list( &l );
-	Split( &l, Stty_command, Whitespace, 0, 0, 0, 1, 0 );
+	l = NEW_OBJ(0,MEMINFO);
+	l = Split_STR_OBJ( /*p*/l, /*nomod*/1, /*str*/Stty_command,
+			/*type*/OBJ_T_LIST, /*linesep*/WHITESPACE, /*escape*/0,
+			/*comment*/ 0, /*trim*/1, /*keysep*/0,/*do_append*/0, /*lc*/0,
+			/*flagvalues*/ 0, /*value_urlunesc*/0, MEMINFO );
 
-	DEBUG4("Do_Do_stty: using TERMIOS, fd %d", fd );
+	DEBUG4("Do_stty: using TERMIOS, fd %d", fd );
 	if( tcgetattr( fd, &t_dat) < 0 ){
 		Errorcode = JABORT;
-		logerr_die("cannot get tty parameters");
+		LOGERR_DIE(LOG_INFO)"cannot get tty parameters");
 	}
 	DEBUG4("Do_stty: before iflag 0x%x, oflag 0x%x, cflag 0x%x lflag 0x%x",
 			 t_dat.c_iflag, t_dat.c_oflag, t_dat.c_cflag, t_dat.c_lflag);
-	for( argcount = 0; argcount < l.count; ++argcount ){
-		arg = l.list[argcount];
+	for( argcount = 0; argcount < LEN_LIST_OBJ(l); ++argcount ){
+		arg = GET_ENTRY_LIST_OBJ(l,argcount);
 		for( i = 0; bauds[i].string && strcasecmp( bauds[i].string, arg); i++);
 		if( bauds[i].string ){
 #ifdef HAVE_CFSETISPEED
-			DEBUG4("Do_Do_stty: using cfsetispeed/cfsetospeed, speed %s, spd %d",
+			DEBUG4("Do_stty: using cfsetispeed/cfsetospeed, speed %s, spd %d",
 				arg, bauds[i].speed );
 			/* POSIX baudrate manipulation */
 			cfsetispeed( &t_dat, bauds[i].speed);
 			cfsetospeed( &t_dat, bauds[i].speed);
 #else
-			DEBUG4("Do_Do_stty: setting tdat.c_cflag");
+			DEBUG4("Do_stty: setting tdat.c_cflag");
 			t_dat.c_cflag &= ~CBAUD;
 			t_dat.c_cflag |= bauds[i].speed;
 #endif
@@ -862,11 +871,11 @@ void Do_stty( int fd, char *Stty_command )
 		}
 		for( i = 0; special[i].name && strcasecmp( special[i].name, arg); i++);
 		if( special[i].name ){
-			if( ++argcount < l.count ){
-				parm = l.list[argcount];
+			if( ++argcount < LEN_LIST_OBJ(l) ){
+				parm = GET_ENTRY_LIST_OBJ(l,argcount);
 			} else {
 				Errorcode = JABORT;
-				fatal(  "Do_stty: missing parameter for %s\n", arg);
+				FATAL(LOG_INFO)  "Do_stty: missing parameter for %s\n", arg);
 			}
 			if( parm[0] == '^') {
 				if( parm[1] == '?' ){
@@ -880,7 +889,7 @@ void Do_stty( int fd, char *Stty_command )
 			DEBUG4("Do_stty: special %s %s", arg, parm);
 			continue;
 		}
-		logmsg("Do_stty: unknown mode: %s", arg);
+		LOGMSG(LOG_INFO)"Do_stty: unknown mode: %s", arg);
 	}
 
 	if( (t_dat.c_lflag & ICANON) == 0 ){
@@ -894,7 +903,7 @@ void Do_stty( int fd, char *Stty_command )
 
 	if( tcsetattr( fd, TCSANOW, &t_dat) < 0 ){
 		Errorcode = JABORT;
-		logerr_die( "cannot set tty parameters");
+		LOGERR_DIE(LOG_INFO) "cannot set tty parameters");
 	}
 }
 #endif
